package com.narvii.amino

import android.content.Context
import kotlin.properties.ReadWriteProperty
import kotlin.reflect.KProperty

/**
 * Created by Timeszoro on 06/11/2017.
 */
class Preference<T>(val context: Context, val name: String, val default: T) : ReadWriteProperty<Any?, T> {

    val prefs by lazy { context.getSharedPreferences("amino", Context.MODE_PRIVATE) }

    override fun getValue(thisRef: Any?, property: KProperty<*>): T {
        return getPreference(name, default)
    }

    override fun setValue(thisRef: Any?, property: KProperty<*>, value: T) {
        putPreference(name, value)
    }

    private fun <U> getPreference(name: String, default: U): U
            = with(prefs) {
        val res: Any = when (default) {
            is String -> getString(name, default)
            is Float -> getFloat(name, default)
            is Boolean -> getBoolean(name, default)
            is Int -> getInt(name, default)
            is Long -> getLong(name, default)
            else -> throw IllegalArgumentException("this type is not support")
        }
        res as U
    }

    private fun <U> putPreference(name: String, value: U) = with(prefs.edit()) {
        when (value) {
            is String -> putString(name, value)
            is Boolean -> putBoolean(name, value)
            is Float -> putFloat(name, value)
            is Int -> putInt(name, value)
            is Long -> putLong(name, value)
            else -> throw IllegalArgumentException("this type is not supported")
        }.apply()
    }

}